/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <stdlib.h>
#include <string.h>
#include "dbg.h"
#include "i18n.h"
#include "common.h"
#include <media_content.h>

#include "ExtNameInfo.h"
#include "DirectoryInfo.h"


struct SDirectoryInfo {
	int nCount;

	filter_h filter;
	media_folder_h folder;
};


bool CDirectoryInfo::Create(CNameInfo::EClassType type)
{
	if (m)
		return false;

	m = new SDirectoryInfo;
	if (!m)
		return false;

	if (!CExtNameInfo::Create(CLASS_TYPE_DIRECTORY)) {
		delete m;
		m = NULL;
		return false;
	}

	return true;
}


void CDirectoryInfo::Destroy(void)
{
	if (!m)
		return;

	CExtNameInfo::Destroy();

	delete m;
	m = NULL;
}


bool CDirectoryInfo::SetCount(int nCount)
{
	if (!m)
		return false;

	m->nCount = nCount;

	return true;
}


int CDirectoryInfo::Count(void)
{
	if (!m)
		return 0;

	return m->nCount;
}


bool CDirectoryInfo::Duplicate(CNameInfo *obj)
{
	if (!m)
		return false;

	CDirectoryInfo *dst = (CDirectoryInfo*)obj;

	if (!CExtNameInfo::Duplicate(obj))
		return false;

	if (!SetCount(dst->Count()))
		return false;

	return true;
}


static bool gFolderSize(media_info_h media_h, void *dt)
{
	long long unsigned int size;
	long long unsigned int *total_size;

	if (!dt)
		return false;

	total_size = (long long unsigned int *)dt;

	if (media_info_get_size(media_h, &size) == MEDIA_CONTENT_ERROR_NONE)
		*total_size = *total_size + size;

	return true;
}


bool CDirectoryInfo::SetLoadInfo(filter_h filter, media_folder_h folder)
{
	if (!m)
		return false;

	m->filter = filter;
	m->folder = folder;

	return true;
}


bool CDirectoryInfo::Load(void)
{
	if (!m)
		return false;

	int r;
	bool ret = false;
	do {
		char* str;
		r = media_folder_get_name(m->folder, &str);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get name failed");
			break;
		}
		SetName(str);
		free(str);

		r = media_folder_get_path(m->folder, &str);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get path failed");
			break;
		}
		SetPath(str);
		free(str);

		r = media_folder_get_folder_id(m->folder, &str);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get id failed");
			break;
		}
		SetId(str);
		free(str);

		int count = 0;
		r = media_folder_get_media_count_from_db(Id(), m->filter, &count);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get id failed");
			break;
		}
		SetCount(count);

		SetType(E_GRP_FOLDER);

		TTime time;
		r = media_folder_get_modified_time(m->folder, &time);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get mod time failed");
			break;
		}
		SetModifieidTime(time);

		TSize size;
		r = media_folder_foreach_media_from_db(Id(), m->filter, gFolderSize, &size);
		if (r != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("Media folder get size failed");
			break;
		}

		SetSize(size);

		ret = true;
	} while (0);

	return ret;
}
