/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <stdlib.h>
#include <string.h>
#include "dbg.h"
#include "i18n.h"
#include "common.h"
#include <media_content.h>

#include "ExtNameInfo.h"
#include "FileInfo.h"


struct SFileInfo {
	media_info_h handle;
};


bool CFileInfo::Create(void)
{
	if (m)
		return false;

	m = new SFileInfo;
	if (!m)
		return false;

	memset(m, 0, sizeof(SFileInfo));

	if (!CExtNameInfo::Create(CExtNameInfo::CLASS_TYPE_FILE)) {
		delete m;
		m = NULL;
		return false;
	}

	return true;
}


void CFileInfo::Destroy(void)
{
	if (!m)
		return;

	CExtNameInfo::Destroy();

	delete m;
	m = NULL;
}


groups gType(media_content_type_e type)
{
	groups ret = E_GRP_OTHER;

	switch (type) {
	case MEDIA_CONTENT_TYPE_IMAGE:
		ret = E_GRP_PHOTO;
		break;

	case MEDIA_CONTENT_TYPE_VIDEO:
		ret = E_GRP_VIDEO;
		break;
	case MEDIA_CONTENT_TYPE_MUSIC:
		ret = E_GRP_MUSIC;
		break;
	default:
		ret = E_GRP_OTHER;
		break;
	}

	return ret;
}


bool CFileInfo::Duplicate(CNameInfo* obj)
{
	if (!m)
		return false;

	if (!CExtNameInfo::Duplicate(obj))
		return false;

	return true;
}


bool CFileInfo::SetLoadInfo(media_info_h handle)
{
	if (!m)
		return false;

	m->handle = handle;
	return true;
}

bool CFileInfo::Load(void)
{
	if (!m)
		return false;

	bool ret = false;
	do {
		char* str;
		if (media_info_get_media_id(m->handle, &str) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media ID Fetch error");
			break;
		}
		SetId(str);
		free(str);

		if (media_info_get_title(m->handle, &str) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media name Fetch error");
			break;
		}
		SetName(str);
		free(str);

		media_content_type_e ctype;
		if (media_info_get_media_type(m->handle, &ctype) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media type Fetch error");
			break;
		}
		SetType(gType(ctype));

		TSize size;
		if (media_info_get_size(m->handle, &size) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media size Fetch error");
			break;
		}
		SetSize(size);

		if (media_info_get_file_path(m->handle, &str) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media file path Fetch error");
			break;
		}
		SetPath(str);
		free(str);

		if (media_info_get_thumbnail_path(m->handle, &str) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media thumb Fetch error");
			break;
		}
		if(str == NULL)
		{
			_ERR("Fail to get string by media_info_get_thumbnail_path");
			break;
		}
		SetThumbnailPath(str);
		free(str);

		TTime time;
		if (media_info_get_modified_time(m->handle, &time) != MEDIA_CONTENT_ERROR_NONE) {
			_ERR("media mod time Fetch error");
			break;
		}
		SetModifieidTime(time);

		ret = true;
	} while (0);

	return ret;
}
