/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <stdio.h>
#include <stdlib.h>
#include <Eina.h>
#include "dbg.h"
#include "common.h"
#include "i18n.h"

#include <media_content.h>
#include "AppCommon.h"
#include "ExtNameInfo.h"
#include "FileList.h"

int CFileList::t_Max(void)
{
	return m_max;
}

void CFileList::t_Remove(Eina_List** list)
{
	_DBG("");
	if (*list == NULL)
		return;

	Eina_List* filelist = *list;
	CExtNameInfo *pInfo;
	void* obj;

	EINA_LIST_FREE(filelist, obj) {
		pInfo = (CExtNameInfo *)obj;
		pInfo->Destroy();
		delete pInfo;
	}

	*list = NULL;
}

CFileList::CFileList(int max)
{
	m_lists = new Eina_List*[max + 1];
	for (int a = 0; a <= max; a++)
		m_lists[a] = NULL;

	m_max = max;
}

CFileList::~CFileList()
{
	RemoveAll();
	delete[] m_lists;
	m_lists = NULL;
}

void CFileList::Add(int idx, void* obj)
{
	ASSERT(0 <= idx && idx <= t_Max());
	m_lists[idx] = eina_list_append(m_lists[idx], obj);
}

Eina_List* CFileList::Clone(int idx)
{
	ASSERT(0 <= idx && idx <= t_Max());
	return eina_list_clone(m_lists[idx]);
}

void CFileList::RemoveAll(void)
{
	for (int a = 0; a <= t_Max(); a++)
		t_Remove(&m_lists[a]);
}
