/*
 * Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>
#include <app.h>
#include <Elementary.h>
#include <Eina.h>
#include "dbg.h"
#include "define.h"
#include "BaseApp.h"

#include <BaseView.h>
#include <ViewMgr.h>
#include <CtxPopup.h>
#include <InputHandler.h>

#include "FbBaseView.h"
#include <ExtNameInfo.h>
#include "FbContextView.h"

#define PARAM_SOURCE "source"
#define FBR_WIN_TITLE "File Browser"

Evas_Object *_add_win(const char *name)
{
	Evas_Object *win;

	win = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (!win)
		return NULL;

	elm_win_title_set(win, FBR_WIN_TITLE);
	elm_win_focus_highlight_enabled_set(win, EINA_TRUE);
	elm_win_focus_highlight_style_set(win, FBR_STYLE_WIN_FOCUS);
	elm_config_focus_move_policy_set(ELM_FOCUS_MOVE_POLICY_CLICK);
	evas_object_show(win);

	return win;
}

class CApp : public CBaseApp {

	struct _appdata {
		const char   *name;
		Evas_Object  *win;
		char         *source;
	}   ad;

	CFbBaseView  *pFbBaseView;
	CContextView *pContextView;
protected:

	virtual bool OnCreate(void)
	{
		Evas_Object *win;

		elm_theme_overlay_add(NULL, THEMEFILE);

		win = _add_win(ad.name);
		if (!win)
			return false;

		ad.win = win;
		
		return true;
	}

	virtual void OnTerminate(void)
	{
		CViewMgr::Finalize();

		if (ad.win)
			evas_object_del(ad.win);

		free(ad.source);
	}

	virtual void OnService(service_h service)
	{
		char *source;
		int r;
		CViewMgr* viewmgr = NULL;

		_DBG("  ******************* OnService *********************");
		_DBG("222");
		elm_win_activate(ad.win);

		r = service_get_extra_data(service, PARAM_SOURCE, &source);
		if (r != SERVICE_ERROR_NONE)
			source = NULL;

		if (source) {
			ad.source = strdup(source);
			free(source);
		}

		CViewMgr::Initialize(ad.win,NULL);
		viewmgr = CViewMgr::GetInstance();
		if(!viewmgr) {
			_ERR("Fail to get viewmgr instance");
			goto error;
		}

		pFbBaseView = new CFbBaseView(FBR_BASE_VIEW);
		if(!viewmgr->AddView(pFbBaseView))
		{
			_ERR("Fail to add pFbBaseView");
			goto error;
		}

		pContextView = new CContextView(FBR_CONTEXT_VIEW);
		if(!viewmgr->AddView(pContextView))
		{
			_ERR("Fail to add pContextView");
			goto error;
		}

		if (!viewmgr->PushView(FBR_BASE_VIEW, ad.source)) {
			_ERR("Fail to push view");
			goto error;
		}
		return;

	error:
		CViewMgr::Finalize();
		elm_exit();
	}

public:

	virtual int Run(int argc, char **argv) {
		memset(&ad, 0x00, sizeof(_appdata));
		ad.name = PACKAGE;

		return CBaseApp::Run(argc, argv);
	}
};

int main(int argc, char **argv)
{
	CApp app;
	app.Run(argc, argv);

	return 0;
}
