
#include <Elementary.h>
#include <AppCommon.h>
#include <BaseView.h>
#include <ViewMgr.h>
#include <BaseLayout.h>
#include <LayoutMgr.h>

#include "dbg.h"
#include "define.h"
#include "FileGrid.h"

#include <CtxPopup.h>
#include "FbBaseView.h"
#include "FbBaseLayout.h"


struct SFbBaseLayout {
	Evas_Object *eoWin;
	Evas_Object *eoBase;
	CFileGrid  *pFileGrid;

	CLayoutMgr *pLayoutMgr;
	CViewMgr   *pViewMgr;
};


void CFbBaseLayout::sm_CbUpdateEmptyFocusSequence(void* cookie)
{
	_DBG();
	CFbBaseView *pfbBaseView = NULL;
	Evas_Object *eoBase = (Evas_Object*)cookie;

	if (!eoBase)
		return;

	pfbBaseView = (CFbBaseView*)evas_object_data_get(eoBase, BASE_VIEW_DATA);
	ASSERT(pfbBaseView);
	pfbBaseView->UpdateEmptyFocusSequence();
}


void CFbBaseLayout::sm_CbUpdatedGrid(void* cookie)
{
	_DBG();
	CFbBaseView *pfbBaseView = NULL;
	Evas_Object *eoBase = (Evas_Object*)cookie;
	if (!eoBase)
		return;

	pfbBaseView = (CFbBaseView*)evas_object_data_get(eoBase, BASE_VIEW_DATA);
	ASSERT(pfbBaseView);
	pfbBaseView->UpdatedGrid();
}


void CFbBaseLayout::sm_CbUnsetFocus(void* cookie)
{
	_DBG();
	CFbBaseView *pfbBaseView = NULL;
	Evas_Object *eoBase = (Evas_Object*)cookie;
	if (eoBase) {
		pfbBaseView = (CFbBaseView*)evas_object_data_get(eoBase, BASE_VIEW_DATA);
		ASSERT(pfbBaseView);
		pfbBaseView->UnsetFocus();
	}
}


void CFbBaseLayout::sm_CbLinkApp(void* cookie, int type, int source_type, char *path)
{
	_DBG();
	CFbBaseView *pfbBaseView = NULL;
	Evas_Object *eoBase = (Evas_Object*)cookie;
	if (eoBase)
	{
		pfbBaseView = (CFbBaseView*)evas_object_data_get(eoBase, BASE_VIEW_DATA);
		ASSERT(pfbBaseView);
		pfbBaseView->CbLinkApp(type, source_type, path);
	}
}


bool CFbBaseLayout::t_CreateFileGrid(Evas_Object *layout)
{
	m->pFileGrid = new CFileGrid;
	if (!m->pFileGrid)
		return false;

	CFileGrid::SCallback cb;
	cb.cbUpdateEmptyFocusSequence = sm_CbUpdateEmptyFocusSequence;
	cb.cbUpdatedGrid = sm_CbUpdatedGrid;
	cb.cbUnsetFocus = sm_CbUnsetFocus;
	cb.cbLinkApp = sm_CbLinkApp;
	cb.cookie = m->eoBase;
	if (!m->pFileGrid->Create(m->eoWin, layout, &cb)) {
		delete m->pFileGrid;
		m->pFileGrid = NULL;
		return false;
	}
	return true;
}


void CFbBaseLayout::t_DestroyFileGrid(void)
{
	if (!m->pFileGrid)
		return;

	m->pFileGrid->Destroy();
	delete m->pFileGrid;
	m->pFileGrid = NULL;
}


bool CFbBaseLayout::Create(CLayoutMgr *mgr, void *data)
{
	ASSERT(!m);
	ASSERT(mgr);
	
	Evas_Object *layout = NULL;

	_CREATE_BEGIN{
		_CHECK(m = new SFbBaseLayout)
		_CHECK(m->pLayoutMgr = mgr)
		_CHECK(m->pViewMgr = CViewMgr::GetInstance())
		_CHECK(m->eoBase = m->pLayoutMgr->Base())
		_CHECK(m->eoWin = m->pViewMgr->Window())
		_CHECK(layout = elm_layout_add(m->eoBase))
		_CHECK(elm_layout_file_set(layout, EDJEFILE, FB_BASE_LAYOUT))
		_CHECK(t_CreateFileGrid(layout))
		_COMMAND{ elm_object_part_content_set(m->eoBase, FBR_PART_CONTENT, layout); }
		_CHECK(CBaseLayout::Create(layout))

		_WHEN_SUCCESS{}

		_CHECK_FAIL{ CBaseLayout::Destroy(); }
		_CHECK_FAIL{ t_DestroyFileGrid(); }
		_CHECK_FAIL{ /* elm_layout_file_set */ }
		_CHECK_FAIL{ /* elm_layout_add */ }
		_CHECK_FAIL{ /* m->pViewMgr->Window()*/ }
		_CHECK_FAIL{ /* m->pLayoutMgr->Base() */ }
		_CHECK_FAIL{ /* CViewMgr::GetInstace() */ }
		_CHECK_FAIL{ /* m->pLayoutMgr = mgr */ }
		_CHECK_FAIL{ delete m; m = NULL; }
	} _CREATE_END_AND_CATCH{ return false; }

	return true;
}


void CFbBaseLayout::Destroy(void)
{
	ASSERT(m);

	CBaseLayout::Destroy();
	t_DestroyFileGrid();
	delete m;
	m = NULL;
}


void CFbBaseLayout::Update(bool flag)
{
	ASSERT(m);
	_DBG();

	m->pFileGrid->Update(flag);
}


void CFbBaseLayout::SetFocus(Eina_Bool flag)
{
	ASSERT(m);
	_DBG();

	m->pFileGrid->SetFocus(flag);
}


Evas_Object* CFbBaseLayout::Grid(void)
{
	ASSERT(m);
	
	return m->pFileGrid->Grid();
}


void CFbBaseLayout::Action(void)
{
	ASSERT(m);
	
	return m->pFileGrid->Action();
}
