/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "i18n.h"
#include "define.h"

#include "AppCommon.h"
#include "ExtNameInfo.h"
#include "common.h"
#include "dbg.h"
#include "ManagerUsb.h"
#include "SortName.h"
#include "Popup.h"
#include "InputHandler.h"

#include <CtxPopup.h>

#include <BaseView.h>
#include "FbBaseView.h"
#include "Info.h"

#include "SourceCtxPopup.h"
#include "SortCtxPopup.h"

#include <ViewMgr.h>
#include <LayoutMgr.h>
#include <BaseLayout.h>
#include "FbBaseLayout.h"

#define TOTAL_GROUP_BTNS	4
#define TOTAL_TOP_BTNS		2
#define TOTAL_UPDATE_CALLBACKS	2
#define PARAM_URI		"uri"
#define PARAM_SOURCE		"source"

struct _btn_info {
	const char *txt;
	const char *part;
	const char *style;
	const char *icon_path;
};

enum topbtns {
	TOPBTN_SOURCE,
	TOPBTN_SORT,
};

const char *source_arg[] = {
	"all",
	"tv",
	"pUsb"
};

const char *linked_app[] = {
	N_(""),
	N_("org.tizen.gallery-tv-ref"),
	N_("org.tizen.video-player-tv-ref"),
	N_("org.tizen.music-player-tv-ref"),
	N_(""),
};

struct SFbBaseView {
	Evas_Object *eoWin;
	Evas_Object *eoBase;
	Evas_Object *eoBtnGroup[TOTAL_GROUP_BTNS];
	Evas_Object *eoSelectedBtnGroup;//Evas_Object *c_grpbtn;
	Evas_Object *eoBtnSource;
	Evas_Object *eoBtnSort;

	CCtxPopup   *pCtxPopup;

	CPopup      *pPopup;

	CLayoutMgr    *pLayoutMgr;
	CFbBaseLayout *pFbBaseLayout;

	SFbBaseView() {
		int a;

		memset(eoBtnGroup, 0, sizeof(eoBtnGroup));
		pHandlerBase = new CHandlerBase(this);
		for (a = 0; a < TOTAL_TOP_BTNS; a++)
			pHandlerButton[a] = new CHandlerButton(this);
		for (a = 0; a < TOTAL_GROUP_BTNS; a++){
			pHandlerGroup[a] = new CHandlerGroup(this);
		}
	}
	virtual ~SFbBaseView() {
		int a;

		delete pHandlerBase;
		for (a = 0; a < TOTAL_TOP_BTNS; a++)
			delete pHandlerButton[a];
		for (a = 0; a < TOTAL_GROUP_BTNS; a++)
			delete pHandlerGroup[a];
	}
	
	class CHandlerBase : public CListenerMgr, public IKeyDownListener {
		SFbBaseView *m;

	public:
		CHandlerBase(SFbBaseView *ins) : IKeyDownListener(this) { m = ins; }

		virtual void OnKeyDown(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Down *ev) {
			if (!strcmp(ev->keyname, KEY_EXIT))
				elm_exit();
			else if (!strcmp(ev->keyname, KEY_ENTER)) {
				if (elm_object_focus_get(m->eoSelectedBtnGroup)) {
					m->pFbBaseLayout->SetFocus(EINA_TRUE);
				}
			}
		}
	} *pHandlerBase;

	class CHandlerButton : public CListenerMgr, protected IKeyDownListener, IMouseMoveListener, IMouseClickedListener {
	private:
		SFbBaseView *m;
	private:
		static void sm_CbCtxPopupSelected(void* cookie, CCtxPopup* instance, const char* text)
		{
			SFbBaseView* m = (SFbBaseView*)cookie;
			if (!m)
				return;

			_DBG(" instance->Type(): %d", instance->Type());
			switch (instance->Type()) {
			case CCtxPopup::TOPBTN_SORT:
				elm_object_text_set(m->eoBtnSort, text);
				m->pFbBaseLayout->Update(true);
				break;

			case CCtxPopup::TOPBTN_SOURCE:
				elm_object_text_set(m->eoBtnSource, text);
				m->pFbBaseLayout->Update(false);
				break;

			default:
				break;
			}
		}

	public:
		CHandlerButton(SFbBaseView *ins) : IKeyDownListener(this), IMouseMoveListener(this), IMouseClickedListener(this) { m = ins; }

		virtual void OnKeyDown(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Down *ev) {
			if (!strcmp(ev->keyname, KEY_BACK))
				elm_object_focus_set(m->eoSelectedBtnGroup, EINA_TRUE);
		}
		virtual void OnMouseMove(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev) {
			if (!elm_object_focus_get(obj))
				elm_object_focus_set(obj, EINA_TRUE);
		}
		virtual void OnMouseClicked(int id, Evas_Object *obj) {
			CSourceCtxPopup::SCallback cb;
			cb.cookie = m;
			cb.onSelected = sm_CbCtxPopupSelected;

			if (obj == m->eoBtnSource) {
				CSourceCtxPopup* pCtxPopup = new CSourceCtxPopup;
				m->pCtxPopup = pCtxPopup;
				pCtxPopup->Create(m->eoBase, &cb);
			}
			else if (obj == m->eoBtnSort) {
				CSortCtxPopup* pCtxPopup = new CSortCtxPopup;
				m->pCtxPopup = pCtxPopup;
				pCtxPopup->Create(m->eoBase, &cb);
			}
		}
	} *pHandlerButton[TOTAL_TOP_BTNS];

	class CHandlerGroup : public CListenerMgr, IMouseMoveListener, IKeyDownListener, IMouseClickedListener {
	private:
		SFbBaseView *m;
	public:
		CHandlerGroup(SFbBaseView *ins) :
			IMouseMoveListener(this),
			IKeyDownListener(this),
			IMouseClickedListener(this) { m = ins; }

		virtual void OnMouseMove(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev) {
			if (!elm_object_focus_get(obj))
				elm_object_focus_set(obj, EINA_TRUE);
		}
		virtual void OnKeyDown(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Down *ev) {
			if (!strcmp(ev->keyname, KEY_BACK))
				elm_exit();
		}
		virtual void OnMouseClicked(int id, Evas_Object *obj) {
			if (m->eoSelectedBtnGroup == obj)
				return;

			elm_object_signal_emit(m->eoSelectedBtnGroup, FBR_SIGNAL_GROUP_UNSELECTED, "");
			m->eoSelectedBtnGroup = obj;
			elm_object_signal_emit(m->eoSelectedBtnGroup, FBR_SIGNAL_GROUP_SELECTED, "");

			m->pFbBaseLayout->Update(false);
		}
	} *pHandlerGroup[TOTAL_GROUP_BTNS];
};



Evas_Object *_add_button(Evas_Object *box)
{
	Evas_Object *button;

	if (!box)
		return NULL;

	button = elm_button_add(box);
	if (!button)
		return NULL;

	evas_object_size_hint_weight_set(button,
		EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(button);

	return button;
}

void CFbBaseView::UpdateEmptyFocusSequence(void)
{
	int i;
	_DBG("");

	for (i = 0; i < TOTAL_GROUP_BTNS; i++)
		elm_object_focus_next_object_set(m->eoBtnGroup[i], m->eoBtnSort, ELM_FOCUS_RIGHT);

	elm_object_focus_next_object_set(m->eoBtnGroup[E_GRP_ALL], m->eoBtnSort,              ELM_FOCUS_UP);
	elm_object_focus_next_object_set(m->eoBtnSort,              m->eoBtnGroup[E_GRP_ALL], ELM_FOCUS_DOWN);
	elm_object_focus_next_object_set(m->eoBtnSource,               m->eoBtnGroup[E_GRP_ALL], ELM_FOCUS_DOWN);

	if (t_SelectedGroupButton() == m->eoBtnGroup[E_GRP_ALL]) {
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT,     _(NO_FILE_MESSAGE));
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT_SUB, _(NO_FILE_MESSAGE_SUB));
	}
	else if (t_SelectedGroupButton() == m->eoBtnGroup[E_GRP_PHOTO]) {
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT,     _(NO_PHOTO_MESSAGE));
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT_SUB, _(NO_PHOTO_MESSAGE_SUB));
	}
	else if (t_SelectedGroupButton() == m->eoBtnGroup[E_GRP_VIDEO]) {
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT,     _(NO_VIDEO_MESSAGE));
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT_SUB, _(NO_VIDEO_MESSAGE_SUB));
	}
	else if (t_SelectedGroupButton() == m->eoBtnGroup[E_GRP_MUSIC]) {
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT,     _(NO_MUSIC_MESSAGE));
		elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT_SUB, _(NO_MUSIC_MESSAGE_SUB));
	}
}

void CFbBaseView::UnsetFocus(void)
{
	elm_object_focus_set(t_SelectedGroupButton(), EINA_TRUE);
}

void CFbBaseView::CbLinkApp(int type, int source_type, char *path)
{
	bundle *args;
	int r;

	args = bundle_create();
	if (!args) {
		_ERR("unable to create bundle");
		return;
	}

	bundle_add(args, PARAM_SOURCE, source_arg[source_type]);
	bundle_add(args, PARAM_URI, path);

	r = aul_launch_app(linked_app[type], args);
	if (args)
		bundle_free(args);
	if (r < 0) {
		_ERR(" Unable to launch linked app");
		return;
	}

	return;
}

void CFbBaseView::UpdatedGrid(void)
{
	int i;

	for (i = 0; i < TOTAL_GROUP_BTNS; i++)
		elm_object_focus_next_object_set(m->eoBtnGroup[i],
		m->pFbBaseLayout->Grid(), ELM_FOCUS_RIGHT);

	elm_object_focus_next_object_set(m->eoBtnGroup[E_GRP_ALL],
		m->eoBtnSort, ELM_FOCUS_UP);
	elm_object_focus_next_object_set(m->eoBtnSort,
		m->eoBtnGroup[E_GRP_ALL], ELM_FOCUS_LEFT);
	elm_object_focus_next_object_set(m->eoBtnSort,
		m->pFbBaseLayout->Grid(), ELM_FOCUS_DOWN);
	elm_object_focus_next_object_set(m->eoBtnSource,
		m->pFbBaseLayout->Grid(), ELM_FOCUS_DOWN);

	elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT, "");
	elm_object_part_text_set(m->eoBase, FBR_PART_NO_CONTENT_SUB, "");
}

void CFbBaseView::t_CreateTopSession(void)
{
	_DBG();
	int i;
	Evas_Object *btn;
	struct _btn_info btninfo[TOTAL_TOP_BTNS];
	btninfo[TOPBTN_SOURCE].style = FBR_STYLE_SOURCE_BTN;
	btninfo[TOPBTN_SOURCE].part  = FBR_PART_SOURCE_HOVERSEL;
	
	btninfo[TOPBTN_SORT].style = FBR_STYLE_SORT_BTN;
	btninfo[TOPBTN_SORT].part  = FBR_PART_SORT_HOVERSEL;

	elm_object_part_text_set(m->eoBase, FBR_PART_TITLE,
		_(FBR_TEXT_TITLE));

	for (i = 0; i < TOTAL_TOP_BTNS; i++) {
		btn = _add_button(m->eoBase);
		if (!btn) {
			_ERR(" Adding btn failed ");
			return;
		}

		elm_object_style_set(btn, btninfo[i].style);
		elm_object_part_content_set(m->eoBase, btninfo[i].part, btn);

		m->pHandlerButton[i]->Connect(btn);

		if (i == TOPBTN_SOURCE)
			m->eoBtnSource = btn;
		else if (i == TOPBTN_SORT)
			m->eoBtnSort = btn;
	}

	elm_object_text_set(m->eoBtnSource, _(CSourceCtxPopup::SourceText(CInfo::SourceType())));
	elm_object_text_set(m->eoBtnSort, _(CSortCtxPopup::SortText(CInfo::SortType())));
}

void CFbBaseView::t_CreateLeftSession(void)
{
	_DBG();
	Evas_Object *box, *ic;
	int i;
	char buf[MAX_LENGTH];
	static struct _btn_info btninfo[TOTAL_GROUP_BTNS];
	btninfo[E_GRP_ALL].txt = N_("ALL");
	btninfo[E_GRP_ALL].icon_path = FBR_IMAGE_GRPBTN_ALL;
	btninfo[E_GRP_PHOTO].txt = N_("PHOTO");
	btninfo[E_GRP_PHOTO].icon_path = FBR_IMAGE_GRPBTN_PHOTO;
	btninfo[E_GRP_VIDEO].txt = N_("VIDEO");
	btninfo[E_GRP_VIDEO].icon_path = FBR_IMAGE_GRPBTN_VIDEO;
	btninfo[E_GRP_MUSIC].txt = N_("MUSIC");
	btninfo[E_GRP_MUSIC].icon_path = FBR_IMAGE_GRPBTN_MUSIC;

	box = elm_box_add(m->eoBase);
	if (!box)
		return;
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_show(box);

	for (i = 0; i < TOTAL_GROUP_BTNS; i++) {
		m->eoBtnGroup[i] = _add_button(box);
		if (!m->eoBtnGroup[i]) {
			_ERR(" failed to add button to box ");
			continue;
		}

		elm_box_pack_end(box, m->eoBtnGroup[i]);
		elm_object_style_set(m->eoBtnGroup[i], FBR_STYLE_GROUPBTN);
		elm_object_text_set(m->eoBtnGroup[i], _(btninfo[i].txt));
		
		m->pHandlerGroup[i]->Connect(m->eoBtnGroup[i]);
		ic = elm_image_add(m->eoBtnGroup[i]);
		if (!ic)
			continue;
		_DBG("");

		snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR,
			btninfo[i].icon_path);
		elm_image_file_set(ic, buf, NULL);
		elm_object_part_content_set(m->eoBtnGroup[i],
			FBR_PART_ELM_SWALLOWICON, ic);
	}
	_DBG("");

	t_SetSelectedGroupButton(m->eoBtnGroup[E_GRP_ALL]);
	_DBG("");

	elm_object_signal_emit(m->eoBtnGroup[E_GRP_ALL],
		FBR_SIGNAL_GROUP_SELECTED, "");
	elm_object_focus_set(m->eoBtnGroup[E_GRP_ALL], EINA_TRUE);
	elm_object_part_content_set(m->eoBase, FBR_PART_GROUPBTN_BOX, box);
}

void CFbBaseView::t_CreateFullView(void)
{
	t_CreateTopSession();
	t_CreateLeftSession();
	_DBG("UpdateLayout");
	m->pFbBaseLayout->Update(false);
}

void CFbBaseView::t_SetSelectedGroupButton(Evas_Object* obj)
{
	m->eoSelectedBtnGroup = obj;
	int idx = -1;

	int a;
	for (a = 0; a < TOTAL_GROUP_BTNS; a++) {
		if (m->eoBtnGroup[a] == obj)
			idx = a;
	}
	CInfo::SetGroupIndex(idx);
}

Evas_Object* CFbBaseView::t_SelectedGroupButton(void)
{
	return m->eoSelectedBtnGroup;
}

void CFbBaseView::t_OnShow(void)
{
	_DBG();
			
	evas_object_show(m->eoBase);

	CBaseView::t_OnShow();
}

void CFbBaseView::t_OnUpdate(void *data)
{
	_DBG();
	m->pFbBaseLayout->Action();
}

void CFbBaseView::t_OnHide(void)
{
	evas_object_hide(m->eoBase);

	CBaseView::t_OnHide();
}

bool CFbBaseView::Create(void *data)
{
	ASSERT(!m);

	Evas_Object *eoWin = CViewMgr::GetInstance()->Window();
	Evas_Object *eoBase = NULL;

	int source_type = E_ALL;

	_CREATE_BEGIN{
		_CHECK(m = new SFbBaseView)
		_CHECK(eoBase = elm_layout_add(eoWin))
		_CHECK(elm_layout_file_set(eoBase, EDJEFILE, FBR_BASE_VIEW))
		_COMMAND{
			evas_object_size_hint_weight_set(eoBase, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
			elm_win_resize_object_add(eoWin, eoBase);

			/* get the source_type using 'data' string */
			if (data) {
				// Find Index
				int i;
				int arrsize = ARRAY_SIZE(source_arg);
				for (i = 0; i < arrsize; i++) {
					if (source_arg[i] && !strcmp(source_arg[i], (const char*)data)) {
						source_type = i;
						break;
					}
				}
				if (source_type < E_ALL || source_type > E_USB)
					source_type = E_ALL;
				CInfo::SetSourceType(source_type);
				m->eoWin = eoWin;
				m->eoBase = eoBase;
			}
		}
		_CHECK(m->pLayoutMgr = new CLayoutMgr)
		_CHECK(m->pLayoutMgr->Create(eoBase, NULL))
		_CHECK(m->pFbBaseLayout = new CFbBaseLayout(FB_BASE_LAYOUT))
		_CHECK(m->pFbBaseLayout->Create(m->pLayoutMgr, NULL))
		_CHECK(m->pLayoutMgr->AddLayout(m->pFbBaseLayout))
		_CHECK(CUsbConnectionListener::Create())

		_WHEN_SUCCESS{}

		_CHECK_FAIL{ CUsbConnectionListener::Destroy(); }
		_CHECK_FAIL{ m->pLayoutMgr->RemoveLayout(m->pFbBaseLayout); }
		_CHECK_FAIL{ m->pFbBaseLayout->Destroy(); }
		_CHECK_FAIL{ delete m->pFbBaseLayout; m->pFbBaseLayout = NULL; }
		_CHECK_FAIL{ m->pLayoutMgr->Destroy(); }
		_CHECK_FAIL{ delete m->pLayoutMgr; m->pLayoutMgr = NULL; }
		_CHECK_FAIL{ /* elm_layout_file_set*/ }
		_CHECK_FAIL{ evas_object_del(eoBase); }
		_CHECK_FAIL{ delete m; m = NULL; }
	} _CREATE_END_AND_CATCH{ return false; }


	// original create grid
	m->pLayoutMgr->Show(FB_BASE_LAYOUT);
	CInfo::SetSortType(0/*CSort::SORT_NAME_AZ*/);
	evas_object_data_set(eoBase, BASE_VIEW_DATA, this);
	t_CreateFullView();
	elm_object_focus_allow_set(eoBase, EINA_FALSE);
	m->pHandlerBase->Connect(eoBase);
	CBaseView::Create(NULL);
	return true;
}

void CFbBaseView::Destroy(void)
{
	ASSERT(m);

	CBaseView::Destroy();
	CUsbConnectionListener::Destroy();

	m->pLayoutMgr->RemoveLayout(m->pFbBaseLayout);
	m->pFbBaseLayout->Destroy();
	delete m->pFbBaseLayout;

	m->pLayoutMgr->Destroy();
	delete m->pLayoutMgr;

	evas_object_del(m->eoBase);

	delete m;
	m = NULL;
}


Evas_Object* CFbBaseView::Base(void)
{
	ASSERT(m);

	return m->eoBase;
}


void CFbBaseView::OnConnect(void)
{
	m->pPopup = new CPopup;
	if (!m->pPopup)
		return;
	if (!m->pPopup->Create(m->eoBase)) {
		delete m->pPopup;
		m->pPopup = NULL;
		return;
	}

	if (CInfo::SourceType() == E_USB) {
		if (!FlagConnected()) {
			CInfo::SetSourceType(E_ALL);
			elm_object_text_set(m->eoBtnSource,
				_(CSourceCtxPopup::SourceText(CInfo::SourceType())));
		}
	}
}


void CFbBaseView::OnUpdated(const SEntity *entity)
{
	m->pPopup->Destroy();
	delete m->pPopup;
	m->pPopup = NULL;

	if (m->pCtxPopup) {
		m->pCtxPopup->Destroy();
		delete m->pCtxPopup;
		m->pCtxPopup = NULL;
	}
	if (CInfo::SourceType() != E_TV) {
		m->pFbBaseLayout->Update(false);
		elm_object_focus_set(t_SelectedGroupButton(), EINA_TRUE);
	}
}

