/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#ifndef __FILE_GRID_H__
#define __FILE_GRID_H__


#include <InputHandler.h>


class CFileGrid : public CListenerMgr, IMouseMoveListener, IKeyUpListener {
private:
	struct SFileGrid* m;

public:
	struct SCallback {
		void(*cbUpdateEmptyFocusSequence)(void* cookie);
		void(*cbUpdatedGrid)(void* cookie);
		void(*cbUnsetFocus)(void* cookie);
		void(*cbLinkApp)(void* cookie, int type, int source_type, char *path);
		void* cookie;
	};

private:
	static void sm_CbRealized(void *data, Evas_Object *obj, void *event_info);
	static void sm_CbUnrealized(void *data, Evas_Object *obj, void *event_info);

	void m_DeleteContextView(void);
	static void sm_CbSelectItem(void *data, Elm_Object_Item *it, const char *emission, const char *source);

	void m_HandleSelection(void);
	void m_GetMediaList(char *pFolderId);
	void m_GetFolderList(void);

protected:
	void t_CreateFileGrid(void);

	void t_Empty(bool sort_flag);
	void t_Update(char *fid, bool sort_flag);
	void t_Fill(void);

public:
	CFileGrid() : IMouseMoveListener(this), IKeyUpListener(this), m(0) { }
	virtual ~CFileGrid() { if (m) Destroy(); }

	bool Create(Evas_Object* win, Evas_Object* base, const SCallback* cb);
	virtual void Destroy(void);
	void Update(bool sort_flag);

	void SetFocus(Eina_Bool flag);
	Evas_Object* Grid(void);
	void Action(void);

	//!
	virtual void OnMouseMove(int id, Evas *e, Evas_Object *obj, Evas_Event_Mouse_Move *ev);
	//!
	virtual void OnKeyUp(int id, Evas *e, Evas_Object *obj, Evas_Event_Key_Up *ev);
};


#endif /* __FILE_GRID_H__ */
