/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "i18n.h"

#include "AppCommon.h"
#include "define.h"
#include "common.h"
#include "dbg.h"
#include "Popup.h"

#define MAX_POPUP_TIME 10 /* seconds */
#define POPUP_ICON_SIZE 48

struct SPopup {
	Evas_Object *popup;
	Elm_Transit *transit;
};

bool CPopup::Create(Evas_Object* base)
{
	ASSERT(base);

	if (m)
		return false;

	m = new SPopup;
	if (!m)
		return false;

	Evas_Object *popup, *label, *icon;
	double scale;
	char buf[MAX_LENGTH];

	popup = elm_popup_add(base);
	if (!popup)
		return NULL;

	elm_object_style_set(popup, FBR_STYLE_LOADING_POPUP);
	elm_popup_orient_set(popup, ELM_POPUP_ORIENT_TOP_RIGHT);
	elm_popup_timeout_set(popup, MAX_POPUP_TIME);
	evas_object_show(popup);

	label = elm_label_add(popup);
	if (!label)
		goto error;

	elm_object_style_set(label, FBR_STYLE_LOADING_LABEL);
	elm_object_text_set(label, _(FBR_TEXT_LOADING));
	elm_object_content_set(popup, label);
	scale = elm_config_scale_get();

	icon = elm_icon_add(popup);
	if (!icon)
		goto error;

	snprintf(buf, sizeof(buf), "%s/%s", IMAGEDIR, FBR_IMAGE_LOADING);
	elm_image_file_set(icon, buf, NULL);
	elm_image_animated_set(icon, EINA_TRUE);
	elm_image_animated_play_set(icon, EINA_TRUE);
	evas_object_size_hint_min_set(icon, POPUP_ICON_SIZE * scale,
		POPUP_ICON_SIZE * scale);
	elm_object_part_content_set(popup, FBR_POPUP_SWALLOW_ICON, icon);

	m->transit = elm_transit_add();
	if (!m->transit)
		goto error;

	elm_transit_object_add(m->transit, icon);
	elm_transit_effect_rotation_add(m->transit, 0, 360);
	elm_transit_duration_set(m->transit, 1);
	elm_transit_repeat_times_set(m->transit, 5);
	elm_transit_objects_final_state_keep_set(m->transit, EINA_TRUE);
	elm_transit_go(m->transit);

	return true;

error:
	evas_object_del(popup);

	return false;
}

void CPopup::Destroy(void)
{
	if (!m)
		return;

	if (m->transit) {
		elm_transit_del(m->transit);
	}
	evas_object_del(m->popup);

	delete m;
	m = NULL;
}
