/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "i18n.h"
#include "define.h"
#include "common.h"

#include "Info.h"
#include "CtxPopup.h"
#include "SortCtxPopup.h"

const char *sorttext[] = {
	N_("Title A-Z"),
	N_("Title Z-A"),
	N_("Date ↓"),
	N_("Date ↑"),
	N_("Size ↓"),
	N_("Size ↑"),
};

const char *sortbtnids[] = {
	"Title A-Z",
	"Title Z-A",
	"Date Asc",
	"Date Des",
	"Size Asc",
	"Size Des"
};

void CSortCtxPopup::t_OnConfiguration(void)
{
	t_SetList(sorttext, ARRAY_SIZE(sorttext), (int)CInfo::SortType(), CCtxPopup::TOPBTN_SORT, sortbtnids, FBR_PART_SORT_POPUP);
}

void CSortCtxPopup::t_OnBtnClicked(Evas_Object* obj, void* ev)
{
	if (!obj)
		return;

	int index = -1;
	const char *id = (const char*)evas_object_data_get(obj, FBR_BTN_ID);

	if (!id)
		return;

	// Find Index
	int i;
	int arrsize = ARRAY_SIZE(sortbtnids);
	for (i = 0; i < arrsize; i++) {
		if (sortbtnids[i] && !strcmp(sortbtnids[i], id)) {
			index = i;
			break;
		}
	}

	if (index == -1 || index == CInfo::SortType())
		return;

	CInfo::SetSortType(index);
	CCtxPopup::t_OnBtnClicked(obj, ev);

	Destroy(); //_destroy_ctxpopup(bd);
}

const char* CSortCtxPopup::SortText(int index)
{
	return sorttext[index];
}
