/*
* Copyright (c) 2014 Samsung Electronics Co., Ltd All Rights Reserved
*
* Licensed under the Apache License, Version 2.0 (the License);
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an AS IS BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

#include <Elementary.h>
#include <Eina.h>
#include <Ecore.h>
#include <utilX.h>
#include <aul.h>
#include "i18n.h"
#include "define.h"
#include "common.h"

#include "Info.h"
#include "CtxPopup.h"
#include "ManagerUsb.h"
#include "SourceCtxPopup.h"

const char *srcbtnids[] = {
	"ALL",
	"TV",
	"USB"
};

const char *sourcetext[] = {
	N_("All Source"),
	N_("TV"),
	N_("USB"),
};

void CSourceCtxPopup::t_OnConfiguration(void)
{
	int size = ARRAY_SIZE(sourcetext);
	
	CUsbConnectionListener usb;
	usb.Create();
	if (!usb.FlagConnected())
		size--;
	usb.Destroy();
	t_SetList(sourcetext, size, (int)CInfo::SourceType(), CCtxPopup::TOPBTN_SOURCE, srcbtnids, FBR_PART_SOURCE_POPUP);
}


void CSourceCtxPopup::t_OnBtnClicked(Evas_Object* obj, void* ev)
{
	if (!obj)
		return;

	int index = -1;
	const char *id = (const char*)evas_object_data_get(obj, FBR_BTN_ID);

	if (!id)
		return;

	// Find Index
	int i;
	int arrsize = ARRAY_SIZE(srcbtnids);
	for (i = 0; i < arrsize; i++) {
		if (srcbtnids[i] && !strcmp(srcbtnids[i], id)) {
			index = i;
			break;
		}
	}
	if (index == -1 || index == CInfo::SourceType())
		return;

	CInfo::SetSourceType(index);
	CCtxPopup::t_OnBtnClicked(obj, ev);

	Destroy();
}

const char* CSourceCtxPopup::SourceText(int index)
{
	return sourcetext[index];
}
